module Wire

push!(LOAD_PATH, pwd())
using WWUMeshes
using GfxBase
using LinearAlgebra

using Luxor

function cube_to_tris()
    cube = cube_mesh()
    triangles = []
    for tri in cube.triangles
        vertices = [] 
        for v = 1:3
            push!(vertices, Vec4(cube.positions[tri.positions[v]]..., 1.0))
        end
        push!(triangles, vertices)
    end
    triangles
end

function main()

    cw = ch = 400
    Drawing(cw, ch)
    background("white")
    setline(0.5)
    model =[20 0 0 0;
            0 20 0 0;
            0 0 20 0;
            0 0 0 1]

    cam   =[1 0 0 0;
            0 1 0 0;
            0 0 1 0;
            0 0 0 1]

    proj = [1 0 0 0;
            0 1 0 0;
            0 0 1 0;
            0 0 0 1]

    vp   = [1 0 0 (cw-1)/2;
            0 1 0 (ch-1)/2;
            0 0 1 0;
            0 0 0 1]

    # the magic happens right here:
    M = vp*proj*cam*model

    for tri in cube_to_tris()
        # transform each vertex
        v1 = M * tri[1]
        v2 = M * tri[2]
        v3 = M * tri[3]
        v1 = Point(v1[1], v1[2]) # drop z and w!
        v2 = Point(v2[1], v2[2])
        v3 = Point(v3[1], v3[2])
        line(v1, v2, :stroke)
        line(v2, v3, :stroke)
        line(v3, v1, :stroke)

    end
    finish()
    preview()
end


end # module Wire

Wire.main()